#Eneroth Texture Positioning Tools

#Author:	Julia Christina Eneroth, Eneroth3, eneroth3@gmail.com

module Ene_texturePositioning

#Menu
menu = UI.menu("Plugins").add_submenu("贴图定位")
menu.add_item("对其贴图") { self.align_material }
menu.add_item("根据边线对齐贴图") { self.align_material(true) }
menu.add_item("逆时针旋转90°"){ self.rotateTexture(90) }
menu.add_item("顺时针旋转90°"){ self.rotateTexture(-90) }
menu.add_item("自定义旋转角度"){ self.rotateTexture }
menu.add_item("随机定位"){ self.depatternize }

#Toolbar
tb = UI::Toolbar.new "贴图定位"

cmd = UI::Command.new("对齐贴图") { self.align_material }
cmd.large_icon = "align_undefinedEdge.png"
cmd.small_icon = "align_undefinedEdge_small.png"
cmd.tooltip = "对其贴图"
cmd.status_bar_text = "将贴图与面边缘对齐"
tb.add_item cmd

cmd = UI::Command.new("根据边线对齐贴图") { self.align_material(true) }
cmd.large_icon = "align_definedEdge.png"
cmd.small_icon = "align_definedEdge_small.png"
cmd.tooltip = "将贴图对齐于面边线"
cmd.status_bar_text = "将贴图与选中的边线对齐"
tb.add_item cmd

cmd = UI::Command.new("逆时针旋转") { self.rotateTexture(90) }
cmd.large_icon = "rotate_cc.png"
cmd.small_icon = "rotate_cc_small.png"
cmd.tooltip = "逆时针旋转"
cmd.status_bar_text = "将贴图逆时针旋转90°"
tb.add_item cmd

cmd = UI::Command.new("顺时针旋转") { self.rotateTexture(-90) }
cmd.large_icon = "rotate_cw.png"
cmd.small_icon = "rotate_cw_small.png"
cmd.tooltip = "顺时针旋转"
cmd.status_bar_text = "将贴图顺时针旋转90°"
tb.add_item cmd

cmd = UI::Command.new("自定义旋转角度") { self.rotateTexture }
cmd.large_icon = "rotate_custom.png"
cmd.small_icon = "rotate_custom_small.png"
cmd.tooltip = "自定义旋转角度"
cmd.status_bar_text = "将选中面的贴图按照逆时针方向自定义角度旋转"
tb.add_item cmd

cmd = UI::Command.new("随机贴图") { self.depatternize }
cmd.large_icon = "depatternize.png"
cmd.small_icon = "depatternize_small.png"
cmd.tooltip = "随机贴图"
cmd.status_bar_text = "随机移动贴图，以防贴图模式冲突"
tb.add_item cmd

UI.start_timer(0.1, false){ tb.restore }#Use timer as workaround for bug 2902434.

#Align Texture
def self.align_material(definedEdge = false)
	#When definedEdge is false all face's textures are aligned to their own first bounding edge.
	#When true all face's textures are aligned to the one selected edge. Only one edge can then be selected
	
	ss = Sketchup.active_model.selection
	
	if definedEdge then									#Align to predefined (selected) edge

		ssEdges = []									# Put selected edges in there own array
		ss.each do |i|
			next unless i.class == Sketchup::Edge
			ssEdges << i
		end#each
		unless ssEdges[0]								# At least one edge must be selected
			UI.messagebox("必须选择边线再按对齐.")
			return
		end#if
		if ssEdges[1] then								# More than one edge can't be selected
			UI.messagebox("只能对齐到一个边线,选择其中一条然后重试.")
			ss.toggle ssEdges							# Deselect all edges so user can select one (just one) and retry
			return
		end#if
		globalvector = ssEdges[0].line[1]						# Edge to align all faces too
		
	end#if
	
	Sketchup.active_model.start_operation "Align Texture"

	ss.each do |i|										#Loop all selected entities
	
		next unless i.class == Sketchup::Face			# Skip if not face
		next unless i.material							# Skip if no material or no texture
		next unless i.material.texture
		vector = i.edges[0].line[1] unless globalvector	# Align to first bounding edge if no edge is defined
		vector = globalvector if globalvector			# Otherwise use defined edge's vector
		next unless i.normal.perpendicular? vector		# Skip if vector isn't in the plane of the face
		achorPoint = i.edges[0].line[0]					# Define point to rotate around
			
		textureWidth = i.material.texture.width
		vector.length = textureWidth					# Change vector's length to materials width
		
		points = [achorPoint, [0,0,0], [achorPoint[0]+vector[0],achorPoint[1]+vector[1],achorPoint[2]+vector[2]], [1,0,0]]	# Reposition material
		i.position_material(i.material, points, true)
		
	end#each
	
	Sketchup.active_model.commit_operation

end#def

#Rotate Texture
def self.rotateTexture(angle=nil)
	#Prompt angle if not set
	
	unless angle											#Prompt angle if not defined
		input = UI.inputbox(["角度 (度, cc)"], [0], "旋转纹理")
		return unless input
		angle = input[0]
	end#unless
	return unless angle
	
	Sketchup.active_model.start_operation "Rotate Texture"

	ss = Sketchup.active_model.selection
	
	ss.each do |i|											#Loop all selected entities
	
		next unless i.class == Sketchup::Face				# Skip if not face
		next unless i.material								# Skip if no material or no texture
		next unless i.material.texture
		
		tw = Sketchup.create_texture_writer					# Create uv helper to get current material position
		uvh = i.get_UVHelper true, false, tw
		
		trans = Geom::Transformation.rotation i.outer_loop.vertices[0].position, i.normal, angle * Math::PI / 180	#Define rotation
		
		pointPairs = []
		(0..1).each do |j|									# Loop some points around face
			point3d = i.outer_loop.vertices[j].position		#  Selet a 3d point
			point3dRotated = point3d.transform(trans)		#  Rotate 3d pont
			pointPairs << point3dRotated					#  Save model's 3d point to array
			point2d = uvh.get_front_UVQ(point3d)
			pointPairs << point2d							#  Save material's corresponding 2d point to array
		end#each

		i.position_material(i.material, pointPairs, true)	#Set material position (pair up model 3d points with texture 2d point)
		
	end#each

	Sketchup.active_model.commit_operation
	
end#def

#Texture Depatternize
def self.depatternize

	Sketchup.active_model.start_operation "Depatternize"

	ss = Sketchup.active_model.selection
	
	ss.each do |i|											#Loop all selected entities
	
		next unless i.class == Sketchup::Face				# Skip if not face
		next unless i.material								# Skip if no material or no texture
		next unless i.material.texture
		
		tw = Sketchup.create_texture_writer					# Create uv helper to get current material position
		uvh = i.get_UVHelper true, false, tw
		
		vector = i.edges[0].line[1]							# Get a vector in the face's plane
		vector.length = rand(i.material.texture.height+i.material.texture.width)							# Set random length (between 0 and materials height + width)
		trans = Geom::Transformation.rotation i.outer_loop.vertices[0].position, i.normal, rand(Math::PI*2)	# Rotate vector randomly in plane
		vector.transform! trans
		trans = Geom::Transformation.translation vector		# Create translation, move point by vector
		
		pointPairs = []
		(0..1).each do |j|									# Loop some points around face
			point3d = i.outer_loop.vertices[j].position		#  Select a 3d point
			point3dRotated = point3d.transform(trans)		#  Move 3d point
			pointPairs << point3dRotated					#  Save model's 3d point to array
			point2d = uvh.get_front_UVQ(point3d)
			pointPairs << point2d							#  Save material's corresponding 2d point to array
		end#each

		i.position_material(i.material, pointPairs, true)	#Set material position (pair up model 3d points with texture 2d point)
		
	end#each
	
	Sketchup.active_model.commit_operation

end#def

end#module